require 'rubygems'
require 'redcarpet'
require 'rouge'
require 'rouge/plugins/redcarpet'
require 'sass'

module Mdx
  class MdxRender < Redcarpet::Render::XHTML
    include Rouge::Plugins::Redcarpet

    attr_reader :title

    def initialize
      @level = 8
      super
    end

    def header title, level
      if level < @level || !@title
        @title = title
        @level = level
      end
      "<h#{level}>#{title}</h#{level}>"
    end

    def self.load_templates
      @@templates = {}

      begin
        io = File.read(__FILE__).force_encoding(Encoding::UTF_8)
        app, data = io.gsub("\r\n", "\n").split(/^__END__$/, 2)
      rescue Errno::ENOENT
        app, data = nil
      end

      if data
        template = nil
        data.each_line do |line|
          if line =~ /^@@\s*(.*\S)\s*$/
            template = ''
            @@templates[$1.to_sym] = template
          elsif template
            template << line
          end
        end
      end

      @@default_style = Sass::Engine.new(@@templates[:style], syntax: :scss, style: :compressed).to_css
    end

    def postprocess doc
      style = @style || @@default_style
      @@templates[:html] % [@title, style, doc]
    end

    load_templates
  end

  def self.process filename, options = {}
    content = File.read(filename).force_encoding(Encoding::UTF_8)
    md = Redcarpet::Markdown.new(MdxRender.new, fenced_code_blocks: true)
    html = md.render(content)
    File.write("#{File.dirname(filename)}/#{File.basename(filename, '.md')}.html", html)
  end
end

__END__

@@html
<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8"/>
<title>%s</title>
<style type="text/css">
%s</style>
</head>
<body>
<div class="content">
%s</div>
</body>
</html>

@@style
html
{
  font-size: 13px;
  font-family: 'Fira Sans', sans-serif;
}

body
{
  margin: 0px;
  padding: 1em;
}

.content
{
  margin: 0 auto;
  max-width: 50em;
}

h1
{
  font-size: 1.5em;
  line-height: 1.2em;
  margin: 0 0 0.5em 0;
}

h2
{
  font-size: 1.2em;
  line-height: 1.2em;
  margin: 0 0 0.25em 0;
}

h3
{
  font-size: 1em;
  line-height: 1.2em;
  margin: 0px;
}

p
{
  font-size: 1em;
  line-height: 1.5em;
  margin: 0 0 0.6em 0;
  text-align: justify;
}

ul
{
  line-height: 1.5em;
  list-style: disc;
  padding: 0 0 0 1.7em;
  margin: 0 0 0.6em 0;
}

a
{
  color: #36a;
  text-decoration: none;
}

a:hover
{
  text-decoration: underline;
}

pre
{
  font-size: 1em;
  line-height: 1.2em;
  margin: 0 0 0.6em 0;
  padding: 0.25em 0em 0.25em 0.5em;
  border-left: 0.25em solid #ccc;
  white-space: pre-wrap;
}

code
{
  font-family: 'Fira Mono', monospace;
  font-weight: 500;
}

pre code
{
  color: #333;
}

.string, .number, .value
{
  color: #c12;
}

.keyword
{
  color: #8a2;
}

.name
{
  color: #36a;
}

.comment
{
  color: #999;
}

blockquote
{
  margin: 0 0 0.6em 0;
  padding: 0.25em 0em 0.25em 0.5em;
  border-left: 0.25em solid #ccc;
  color: #666;
}

hr
{
  width: 80%;
  margin: 2em auto;
  background: #ccc;
  border: none;
  height: 1px;
}

table
{
  margin: 0 0 0.6em 0;
  padding: 0;
  width: 100%;
  border-collapse: collapse;
}

table thead
{
  border-bottom: 1px solid #333;
}

table th, table td
{
  padding: 0.1em 0.25em;
  text-align: left;
  vertical-align: top;
}

table th
{
  font-weight: bold;
}

table tbody tr:nth-child(even)
{
  background: #eee;
}

p:last-child, ul:last-child, li:last-child, pre:last-child, blockquote:last-child, table:last-child
{
  margin-bottom: 0px;
}

@media print
{
  a:after {
    content: " (" attr(href) ")";
  }
}

$red: #c12;
$blue: #09d;
$green: #8a2;
$yellow: #e91;
$purple: #63a;
$black: #333;
$gray: #999;

.highlight {
  /* Text.Whitespace: Specially highlighted whitespace */
  .w {}
  /* Error: Lexer errors */
  .err {}
  /* Other: Token for data not matched by a parser (e.g. HTML markup in PHP code) */
  .x {}
  /* Keyword: Any keyword */
  .k {color: $red;}
  /* Keyword.Constant: Keywords that are constants */
  .kc {color: $blue;}
  /* Keyword.Declaration: Keywords used for variable declaration (e.g. var in javascript) */
  .kd {color: $blue;}
  /* Keyword.Namespace: Keywords used for namespace declarations */
  .kn {color: $red;}
  /* Keyword.Pseudo: Keywords that aren't really keywords */
  .kp {color: $red;}
  /* Keyword.Reserved: Keywords which are reserved (such as end in Ruby) */
  .kr {color: $red;}
  /* Keyword.Type: Keywords wich refer to a type id (such as int in C) */
  .kt {color: $blue;}
  /* Name: Variable/function names */
  .n {color: $black;}
  /* Name.Attribute: Attributes (in HTML for instance) */
  .na {color: $green;}
  /* Name.Builtin: Builtin names which are available in the global namespace */
  .nb {color: $black;}
  /* Name.Builtin.Pseudo: Builtin names that are implicit (such as self in Ruby) */
  .bp {color: $black;}
  /* Name.Class: For class declaration */
  .nc {color: $blue;}
  /* Name.Constant: For constants */
  .no {color: $blue;}
  /* Name.Decorator: For decorators in languages such as Python or Java */
  .nd {color: $black;}
  /* Name.Entity: Token for entitites such as &nbsp; in HTML */
  .ni {color: $purple;}
  /* Name.Exception: Exceptions and errors (e.g. ArgumentError in Ruby) */
  .ne {color: $blue;}
  /* Name.Function: Function names */
  .nf {color: $blue;}
  /* Name.Property: Token for properties */
  .py {color: $blue;}
  /* Name.Label: For label names */
  .nl {color: $black;}
  /* Name.Namespace: Token for namespaces */
  .nn {color: $red;}
  /* Name.Other: For other names */
  .nx {color: $black;}
  /* Name.Tag: Tag mainly for markup such as XML or HTML */
  .nt {color: $red;}
  /* Name.Variable: Token for variables */
  .nv {color: $black;}
  /* Name.Variable.Class: Token for class variables (e.g. @@var in Ruby) */
  .vc {color: $black;}
  /* Name.Variable.Global: For global variables (such as $LOAD_PATH in Ruby) */
  .vg {color: $black;}
  /* Name.Variable.Instance: Token for instance variables (such as @var in Ruby) */
  .vi {color: $black;}
  /* Literal: Any literal (if not further defined) */
  .l {color: $purple;}
  /* Literal.Date: Date literals */
  .ld {color: $purple;}
  /* Literal.String: String literals */
  .s {color: $yellow;}
  /* Literal.String.Backtick: String enclosed in backticks */
  .sb {color: $yellow;}
  /* Literal.String.Char: Token type for single characters */
  .sc {color: $yellow;}
  /* Literal.String.Doc: Documentation strings (such as in Python) */
  .sd {color: $yellow;}
  /* Literal.String.Double: Double quoted strings */
  .s2 {color: $yellow;}
  /* Literal.String.Escape: Escaped sequences in strings */
  .se {color: $purple;}
  /* Literal.String.Heredoc: For "heredoc" strings (e.g. in Ruby) */
  .sh {color: $yellow;}
  /* Literal.String.Interpol: For interpoled part in strings (e.g. in Ruby) */
  .si {color: $yellow;}
  /* Literal.String.Other: Token type for any other strings (for example %q{foo} string constructs in Ruby) */
  .sx {color: $yellow;}
  /* Literal.String.Regex: Regular expressions literals */
  .sr {color: $yellow;}
  /* Literal.String.Single: Single quoted strings */
  .s1 {color: $yellow;}
  /* Literal.String.Symbol: Symbols (such as :foo in Ruby) */
  .ss {color: $purple;}
  /* Literal.Number: Any number literal (if not further defined) */
  .m {color: $purple;}
  /* Literal.Number.Float: Float numbers */
  .mf {color: $purple;}
  /* Literal.Number.Hex: Hexadecimal numbers */
  .mh {color: $purple;}
  /* Literal.Number.Integer: Integer literals */
  .mi {color: $purple;}
  /* Literal.Number.Integer.Long: Long interger literals */
  .il {color: $purple;}
  /* Literal.Number.Oct: Octal literals */
  .mo {color: $purple;}
  /* Operator: Operators (commonly +, -, /, *) */
  .o {color: $red;}
  /* Operator.Word: Word operators (e.g. and) */
  .ow {color: $red;}
  /* Punctuation: Punctuation which is not an operator */
  .p {color: $black;}
  /* Comment: Single ligne comments */
  .c {color: $gray;}
  /* Comment.Multiline: Mutliline comments */
  .cm {color: $gray;}
  /* Comment.Preproc: Preprocessor comments such as <% %> in ERb */
  .cp {color: $gray;}
  /* Comment.Single: Comments that end at the end of the line */
  .c1 {color: $gray;}
  /* Comment.Special: Special data in comments such as @license in Javadoc */
  .cs {color: $gray;}
  /* Generic: Unstyled token */
  .g {color: $black;}
  /* Generic.Deleted: Token value as deleted */
  .gd {color: $red;}
  /* Generic.Emph: Token value as emphasized */
  .ge {font-style: italic; color: $black;}
  /* Generic.Error: Token value as an error message */
  .gr {color: $red;}
  /* Generic.Heading: Token value as a headline */
  .gh {font-weight: bold; color: $black;}
  /* Generic.Inserted: Token value as inserted */
  .gi {color: $green;}
  /* Generic.Output: Marked as a program output */
  .go {color: $black;}
  /* Generic.Prompt: Marked as a command prompt */
  .gp {color: $black;}
  /* Generic.Strong: Mark the token value as bold (for rst lexer) */
  .gs {font-weight: bold; color: $black;}
  /* Generic.Subheading: Marked as a subheadline */
  .gu {font-weight: bold; color: $black;}
  /* Generic.Traceback: Mark the token as a part of an error traceback */
  .gt {color: $black;}
  /* Generic.Lineno: Line numbers */
  .gl {color: $gray;}
}
